/*

	bmp.h - bitmap support routines for the Power JBIG-2 Decoder / Encoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/

#ifndef BMP_H

#define BMP_H

#include "jb2types.h"

/* Combination Operators defined in JBIG-2 */

#define COMBOPOR 0
#define COMBOPAND 1
#define COMBOPXOR 2
#define COMBOPXNOR 3
#define COMBOPREPLACE 4

extern BYTE ConstMask1[];
extern BYTE ConstMask0[];

/* The BMP type is the base bitmap data type for the entire decoder / encoder
   The structure is designed to be compatible to the Windows BITMAP structure
   ie: the following fields are equivalent:
		*Bits <=> BITMAP.bmBits
		Width <=> BITMAP.bmWidth
		Height <=> BITMAP.bmHeight
		AlignWidthBytes <=> BITMAP.bmWidthBytes

   The constant BMPALIGNBYTES is used to ensure that the memory is 16-bit
   aligned, as required by windows.  For quad or byte aligned data,
   BMPALIGNBYTES can be set to 4 or 1, respectively */

typedef struct tagBMP {
	BYTE *Bits;
	ULONG Width;
	ULONG Height;
	ULONG MinWidthBytes;
	ULONG AlignWidthBytes;
} BMP;

#define BMPALIGNBYTES 2

/* These macros are used to set/get single pixel values in a bitmap
   Note: the "safe" versions should be used if it is possible the call will
   extend beyond the boundaries of the bitmap */

#define GetBitBMP(B,X,Y) ((unsigned char)(((B)->Bits[(X)/8+(Y)*(B)->AlignWidthBytes]&(128>>MOD8(X)))?1:0))
#define SetBitBMP(B,X,Y,V) ((V==0)?((B)->Bits[(X)/8+(Y)*(B)->AlignWidthBytes]&=(unsigned char)(255-(128>>MOD8(X)))):((B)->Bits[(X)/8+(Y)*(B)->AlignWidthBytes]|=(unsigned char)(128>>MOD8(X))))
#define GetBitBMPsafe(B,X,Y) ((unsigned char)((((Y)<0)||((X)<0)||((signed long)(X)>=(signed long)(B)->Width)||((signed long)(Y)>=(signed long)(B)->Height))?0:GetBitBMP(B,X,Y)))
#define SetBitBMPsafe(B,X,Y,V) ((((Y)<0)||((X)<0)||((X)>=(B)->Width)||((Y)>=(B)->Height))?0:SetBitBMP(B,X,Y,V))

DLLEXPORT void CreateBMP(BMP *,ULONG,ULONG,ULONG);
DLLEXPORT void DestroyBMP (BMP *);
DLLEXPORT void SaveBMP(const char *, BMP *);
DLLEXPORT ULONG LoadBMP(const char *, BMP *);

void CombineBMP(BMP *,BMP *,SLONG,SLONG,ULONG);
void ExtractRegion (BMP *, BMP *, ULONG ox, ULONG oy);
void CopyBMP (BMP *New, BMP *Old);
ULONG ExactBMP (BMP *, BMP *);
void ClearRegion (BMP *Image, ULONG ox, ULONG oy, ULONG width, ULONG height);
void printfBMP(BMP *img);
void InvertBMP (BMP *Image);
ULONG AreaBMP(BMP *image);

#endif

