/*

	deccommon.h - decoder support routines for the Power JBIG-2 Decoder

	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/

#ifndef DECCOMMON_H

#define DECCOMMON_H

#include <stdlib.h>

#include "jb2structs.h"

#include "mqcoder.h"
#include "mqint.h"

#include "mmr.h"

#define DecodeBit(pD) DecodeBits(pD,1)
#define DecodeByte(pD) DecodeBits(pD,8)
#define DecodeQuad(pD) DecodeBits(pD,32)

/* deccommon.c */
void InitDecoder(DECODER *, BYTE *, ULONG);
ULONG DecodeBits(DECODER *, ULONG);
void DecodeFlush(DECODER *);
BYTE *DecoderPtr(DECODER *);
BOOLN DecodeEOS(DECODER *);
void DecSkipBytes(DECODER *, SLONG);
SLONG DecodeHuff(DECODER *, HUFFTABLE *tbl, ULONG *bOOB);
BYTE MQMainIn(void *);
ULONG DecodeHuffTree (DECODER *, HUFFNODE *);

/* decgeneric.c */
void DecodeGenericRegionSegment(SEGMENTHEADER *);
void DecodeGenericRegion(GENREGIONPARMS *,BMP *);
void DecodeRefinementRegion(GENREFREGIONPARMS *, BMP *);

/* dechalftone.c */
void DecodeHalftoneDictionarySegment(SEGMENTHEADER *);
void DecodeHalftoneRegionSegment(SEGMENTHEADER *);
void DecodeRefinementRegionSegment(SEGMENTHEADER *);

/* decsegments.c */
void ReadSegmentHeader(DECODER *, SEGMENTHEADER **SegmentHeaders,ULONG *numSegments,ULONG *maxSegments);
void FreeSegmentHeader(SEGMENTHEADER *);
void BuildSegmentTree(SEGMENTHEADER *, ULONG);
void RegionToPage(SEGMENTHEADER *);
void DecodeTableSegment(SEGMENTHEADER *);
void DecodePageSegment(SEGMENTHEADER *);
void DecodeRegionSegment(DECODER *, REGIONSEGMENT *, BOOLN);
void SkipUnknownSegmentLength(DECODER *);
void DecodeSegment (SEGMENTHEADER *);

/* decsymbols.c */
void DecodeSymbolDictionarySegment(SEGMENTHEADER *);
void DecodeSymbolRegionSegment(SEGMENTHEADER *);

/* Output Files */
DLLEXPORT BOOLN JB2ParseFile (JB2FILE *);
DLLEXPORT void JB2DecodeGlobal(JB2FILE *);
DLLEXPORT void JB2DecodePage(JB2PAGE *);
DLLEXPORT void JB2FreeFile(JB2FILE *);

#endif
