/*

	decgeneric.c - Generic & Refinement Coding for the Power JBIG-2 Decoder

	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/

#include "jb2common.h"
#include "deccommon.h"
#include "mqcontext.h"

void DecodeGenericRegion(GENREGIONPARMS *inParms,BMP *GBREG) {

	ULONG LTP;
	ULONG SLTP;
	ULONG CONTEXT;
	ULONG bitval;
	BYTE *Bits;
	ULONG m;
	register ULONG k;
	register ULONG l;
	MQCONTEXT MQContext;
	MQDEC *pMQDec = &inParms->pCoderData->MQDec;
	BYTE *aIndex = inParms->pCoderData->Generic.Index;
	BYTE *aMPS = inParms->pCoderData->Generic.MPS;

	if (inParms->MMR) {
		MMRDecode(&inParms->pCoderData->Dec, &inParms->pCoderData->MMRTables, GBREG, inParms->bUnknownMMRLength);
		DecodeFlush(&inParms->pCoderData->Dec);
	} else {
		InitContext(&MQContext,GBREG,inParms->GBTEMPLATE,inParms->GBATX,inParms->GBATY);
		LTP = 1;
		for (k = 0; k < GBREG->Height; k++) {
			NextContextRow(&MQContext);
			bitval=0;
			Bits = &GBREG->Bits[k*GBREG->AlignWidthBytes];
			m = 0;
			if (inParms->TPDON) {
				switch (inParms->GBTEMPLATE) {
					case 0: CONTEXT = 0x9B25; break;
					case 1: CONTEXT = 0x0795; break;
					case 2: CONTEXT = 0x00E5; break;
					case 3: CONTEXT = 0x0195; break;
				}
				SLTP = MQDecode(pMQDec,&aIndex[CONTEXT],&aMPS[CONTEXT]);
				LTP = LTP ^ SLTP;
			}
			if (LTP==0) {
				if (k==0)
					for (l = 0; l < GBREG->MinWidthBytes; l++)
						GBREG->Bits[k * GBREG->AlignWidthBytes + l] = 0;
				else
					for (l = 0; l < GBREG->MinWidthBytes; l++)
						GBREG->Bits[k * GBREG->AlignWidthBytes + l] = GBREG->Bits[(k-1) * GBREG->AlignWidthBytes + l];
			} else {
				for (l = 0; l < GBREG->Width; l++) {
					CONTEXT = GetContext(&MQContext,bitval);
					if (inParms->USESKIP) {
						if (GetBitBMP(inParms->pSKIP,l,k))
							bitval = 0;
						else
							bitval = MQDecode(pMQDec,&aIndex[CONTEXT],&aMPS[CONTEXT]);
					} else
						bitval = MQDecode(pMQDec,&aIndex[CONTEXT],&aMPS[CONTEXT]);
					
					if (bitval)
						(*Bits) |= ConstMask1[m];
					else
						(*Bits) &= ConstMask0[m];

					m++;
					if (m==8) {
						m = 0;
						Bits++;
					}

				}
			}
		}
	}
}


void DecodeGenericRegionSegment(SEGMENTHEADER *Seg) {

	GENREGIONPARMS GRParms;
	ULONG flags;

	DECODER *pDec = &Seg->pCoderData->Dec;

	DecodeRegionSegment(pDec,&Seg->RegSegment,Seg->bUnknownLength);
	CreateBMP(&Seg->Region,Seg->RegSegment.Width,Seg->RegSegment.Height,0);

	GRParms.pCoderData = Seg->pCoderData;

	flags = DecodeByte(pDec);
	GRParms.MMR = getBIT(flags,0);
	GRParms.GBTEMPLATE = getBITS(flags,1,2);
	if ((GRParms.GBTEMPLATE)&&(GRParms.MMR))
		LogWarning("Invalid GBTEMPLATE");
	GRParms.TPDON = getBIT(flags,3);
	if (getBITS(flags,4,7))
		LogWarning("Invalid Reserved Bits");

	if (GRParms.MMR==0) {
		GRParms.GBATX[0] = (signed char) DecodeByte(pDec);
		GRParms.GBATY[0] = (signed char) DecodeByte(pDec); 
		if (GRParms.GBTEMPLATE==0) {
			GRParms.GBATX[1] = (signed char) DecodeByte(pDec); 
			GRParms.GBATY[1] = (signed char) DecodeByte(pDec); 
			GRParms.GBATX[2] = (signed char) DecodeByte(pDec); 
			GRParms.GBATY[2] = (signed char) DecodeByte(pDec); 
			GRParms.GBATX[3] = (signed char) DecodeByte(pDec); 
			GRParms.GBATY[3] = (signed char) DecodeByte(pDec); 
		}
		MQInitDec(&GRParms.pCoderData->MQDec);
		ResetMQBuffer(&GRParms.pCoderData->Generic,MQCONTEXTMAX);
	}
	
	GRParms.USESKIP = 0;
	
	GRParms.bUnknownMMRLength = 0;

	GRParms.pCoderData = Seg->pCoderData;

	DecodeGenericRegion(&GRParms,&Seg->Region);

	if (Seg->Type != 36) {
		RegionToPage(Seg);
		DestroyBMP(&Seg->Region);
	}
}

void DecodeRefinementRegion(GENREFREGIONPARMS *inParms, BMP *GRREG) {

	ULONG CONTEXT;
	ULONG SLTP;
	BYTE *Bits;
	ULONG m;

	ULONG bitval;

	ULONG k;
	ULONG l;

	ULONG LTP = 0;

	MQCONTEXT MQContext;
	MQDEC *pMQDec = &inParms->pCoderData->MQDec;
	BYTE *aIndex = inParms->pCoderData->Refine.Index;
	BYTE *aMPS = inParms->pCoderData->Refine.MPS;

	InitRefContext(&MQContext,inParms->GRREFERENCE,GRREG,inParms->GRREFERENCEDX,inParms->GRREFERENCEDY, inParms->GRTEMPLATE, inParms->GRATX, inParms->GRATY);

	for (k = 0; k < GRREG->Height; k++) {
		NextContextRow(&MQContext);
		bitval=0;
		Bits = &GRREG->Bits[k*GRREG->AlignWidthBytes];
		m = 0;
		if (inParms->TPRON) {
			switch (inParms->GRTEMPLATE) {
				case 0: CONTEXT = 0x0100; break;
				case 1: CONTEXT = 0x0080; break;
			}
			SLTP = MQDecode(pMQDec,&aIndex[CONTEXT],&aMPS[CONTEXT]);
			LTP = LTP ^ SLTP;
		}
		for (l = 0; l < GRREG->Width; l++) {
			CONTEXT = GetContext(&MQContext,bitval);
			if (LTP==0)
				bitval = MQDecode(pMQDec,&aIndex[CONTEXT],&aMPS[CONTEXT]);
			else
				if (Check3x3(inParms->GRREFERENCE,l-inParms->GRREFERENCEDX,k-inParms->GRREFERENCEDY))
					bitval = GetBitBMP(inParms->GRREFERENCE,l-inParms->GRREFERENCEDX,k-inParms->GRREFERENCEDY);
				else 
					bitval = MQDecode(pMQDec,&aIndex[CONTEXT],&aMPS[CONTEXT]);

			if (bitval)
				(*Bits) |= ConstMask1[m];
			else
				(*Bits) &= ConstMask0[m];

			m++;
			if (m==8) {
				m = 0;
				Bits++;
			}
		}

	}
}


void DecodeRefinementRegionSegment(SEGMENTHEADER *Seg) {

	GENREFREGIONPARMS GRRParms;
	ULONG flags;

	BMP PageRefinement;

	DECODER *pDec = &Seg->pCoderData->Dec;

	DecodeRegionSegment(pDec,&Seg->RegSegment,Seg->bUnknownLength);
	CreateBMP(&Seg->Region,Seg->RegSegment.Width,Seg->RegSegment.Height,0);

	GRRParms.pCoderData = Seg->pCoderData;

	if (Seg->bRefIntermediate)
		GRRParms.GRREFERENCE = &Seg->pINTSegment->Region;
	else {
		CreateBMP(&PageRefinement,Seg->RegSegment.Width,Seg->RegSegment.Height,0);
		ExtractRegion(&PageRefinement,Seg->pPageImage,Seg->RegSegment.OffsetX,Seg->RegSegment.OffsetY);
		GRRParms.GRREFERENCE = &PageRefinement;
	}
			
	flags = DecodeByte(pDec);

	GRRParms.GRTEMPLATE = getBIT(flags,0);
	GRRParms.TPRON = getBIT(flags,1);
	if (getBITS(flags,2,7))
		LogWarning("Invalid Reserved Bits");

	if (GRRParms.GRTEMPLATE==0) {
		GRRParms.GRATX[0] = (signed char) DecodeByte(pDec);
		GRRParms.GRATY[0] = (signed char) DecodeByte(pDec);
		GRRParms.GRATX[1] = (signed char) DecodeByte(pDec);
		GRRParms.GRATY[1] = (signed char) DecodeByte(pDec);
	}
	
	GRRParms.GRREFERENCEDX = 0;
	GRRParms.GRREFERENCEDY = 0;

	MQInitDec(&GRRParms.pCoderData->MQDec);
	ResetMQBuffer(&GRRParms.pCoderData->Refine,MQCONTEXTMAX);

	DecodeRefinementRegion(&GRRParms,&Seg->Region);

	if (Seg->Type != 40) {
		RegionToPage(Seg);
		DestroyBMP(&Seg->Region);
	}

	DestroyBMP(GRRParms.GRREFERENCE);
}

