/*

	dechalftone.c - Halftone Coding for the Power JBIG-2 Decoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/

#include "jb2common.h"
#include "deccommon.h"

static void DecodeHalftoneDictionary(HALFTONEDICTPARMS *inParms, HALFTONETABLE *TBL) {

	ULONG GRAY;
	GENREGIONPARMS GRParms;
	BMP Bhdc;

	GRParms.pCoderData = inParms->pCoderData;

	CreateBMP(&Bhdc,TBL->Size * TBL->PatWidth,TBL->PatHeight,0);

	GRParms.MMR = inParms->HDMMR;
	GRParms.GBTEMPLATE = inParms->HDTEMPLATE;
	GRParms.TPDON = 0;
	GRParms.USESKIP = 0;
	GRParms.GBATX[0] = - (SLONG) TBL->PatWidth;
	GRParms.GBATY[0] = 0;
	GRParms.GBATX[1] = -3;
	GRParms.GBATY[1] = -1;
	GRParms.GBATX[2] = 2;
	GRParms.GBATY[2] = -2;
	GRParms.GBATX[3] = -2;
	GRParms.GBATY[3] = -2;
	GRParms.bUnknownMMRLength = 0;

	DecodeGenericRegion(&GRParms,&Bhdc);

	GRAY=0;

	for (GRAY=0;GRAY < TBL->Size; GRAY++) {
		CreateBMP(&TBL->aPatterns[GRAY],TBL->PatWidth,TBL->PatHeight,0);
		ExtractRegion(&TBL->aPatterns[GRAY],&Bhdc,GRAY * TBL->PatWidth, 0);
	}

	DestroyBMP(&Bhdc);
}

static void DecodeGrayRegion (GRAYREGIONPARMS *inParms, ULONG *GSVALS) {

	BMP *GSPLANES;
	SLONG j;

	ULONG x,y;
	GENREGIONPARMS GRParms;

	GSPLANES = jb2malloc(inParms->GSBPP * sizeof(*GSPLANES));

	GRParms.pCoderData = inParms->pCoderData;

	GRParms.MMR = inParms->GSMMR;
	GRParms.GBTEMPLATE = inParms->GSTEMPLATE;
	GRParms.TPDON = 0;
	GRParms.USESKIP = inParms->GSUSESKIP;
	GRParms.pSKIP = inParms->GSKIP;
	if (inParms->GSTEMPLATE <=1)
		GRParms.GBATX[0] = 3;
	else
		GRParms.GBATX[0] = 2;
	GRParms.GBATY[0] = -1;
	GRParms.GBATX[1] = -3;
	GRParms.GBATY[1] = -1;
	GRParms.GBATX[2] = 2;
	GRParms.GBATY[2] = -2;
	GRParms.GBATX[3] = -2;
	GRParms.GBATY[3] = -2;
	GRParms.bUnknownMMRLength = 1;

	if (GRParms.MMR==0)
		MQInitDec(&GRParms.pCoderData->MQDec);

	for (j=inParms->GSBPP-1;j>=0;j--) {
		
		CreateBMP(&GSPLANES[j],inParms->GSW,inParms->GSH,0);
		
		DecodeGenericRegion(&GRParms,&GSPLANES[j]);
		
		if (j < (SLONG)(inParms->GSBPP - 1))
			for (y=0;y<GSPLANES[j].Height;y++)
				for (x=0;x<GSPLANES[j].Width;x++) 
					SetBitBMP(&GSPLANES[j],x,y,((GetBitBMP(&GSPLANES[j],x,y))^(GetBitBMP(&GSPLANES[j+1],x,y))));
	}

	for (y=0;y<inParms->GSH;y++)
		for (x=0;x<inParms->GSW;x++) {
			GSVALS[x + y * inParms->GSW] = 0;
			for (j=inParms->GSBPP-1;j>=0;j--) {
				GSVALS[x + y * inParms->GSW] <<= 1;
				GSVALS[x + y * inParms->GSW] += GetBitBMP(&GSPLANES[j],x,y);
			}
		}

	for (j=inParms->GSBPP-1;j>=0;j--)
		DestroyBMP(&GSPLANES[j]);

	jb2free(GSPLANES);

}

static void DecodeHalftoneRegion(HALFTONEREGIONPARMS *inParms, BMP *HTREG) {

	ULONG Ng;
	ULONG Mg;
	SLONG x;
	SLONG y;
	BMP HSKIP;
	ULONG HBPP;
	ULONG *GI; 

	GRAYREGIONPARMS GYRParms;

	GYRParms.pCoderData = inParms->pCoderData;

	if (inParms->HDEFPIXEL)
		InvertBMP(HTREG);

	if (inParms->HENABLESKIP) {
		CreateBMP(&HSKIP, HTREG->Width, HTREG->Height,0);

		for(Mg=0;Mg<inParms->HGH;Mg++)
			for (Ng=0;Ng<inParms->HGW;Ng++) {
				x = (inParms->HGX + Mg * inParms->HRY + Ng * inParms->HRX);
				if (x < 0) {
					x >>= 8;
					x |= 0xF0;
				} else
					x >>= 8;

				y = (inParms->HGY + Mg * inParms->HRX - Ng * inParms->HRY);
				if (y < 0) {
					y >>= 8;
					y |= 0xF0;
				} else
					y >>= 8;

				if ((x+inParms->HPATS->PatWidth<=0)||(x>= (SLONG) HTREG->Width)||(y+inParms->HPATS->PatHeight<=0)||(y>= (SLONG) HTREG->Height))
					SetBitBMP(&HSKIP,Ng,Mg,1);
			}
	}

	
	BITSREQUIRED(inParms->HPATS->Size,HBPP);

	GI = jb2calloc(inParms->HGW * inParms->HGH,sizeof(*GI));

	GYRParms.GSMMR = inParms->HMMR;
	GYRParms.GSW = inParms->HGW;
	GYRParms.GSH = inParms->HGH;
	GYRParms.GSBPP = HBPP;
	GYRParms.GSUSESKIP = inParms->HENABLESKIP;
	GYRParms.GSKIP = &HSKIP;
	GYRParms.GSTEMPLATE = inParms->HTEMPLATE;

	DecodeGrayRegion(&GYRParms,GI);

	for(Mg=0;Mg<inParms->HGH;Mg++)
		for (Ng=0;Ng<inParms->HGW;Ng++) {
			x = (inParms->HGX + Mg * inParms->HRY + Ng * inParms->HRX);
			if (x < 0) {
				x >>= 8;
				x |= 0xFF000000;
			} else
				x >>= 8;

			y = (inParms->HGY + Mg * inParms->HRX - Ng * inParms->HRY);
			if (y < 0) {
				y >>= 8;
				y |= 0xFF000000;
			} else
				y >>= 8;

			CombineBMP(HTREG,&inParms->HPATS->aPatterns[GI[Ng + Mg * inParms->HGW]],x,y,inParms->HCOMBOP);
		}

	
	if (inParms->HENABLESKIP)
		DestroyBMP(&HSKIP);
		
	jb2free(GI);
	
}

void DecodeHalftoneDictionarySegment(SEGMENTHEADER *Seg) {

	HALFTONEDICTPARMS HTParms;
	ULONG flags;

	DECODER *pDec = &Seg->pCoderData->Dec;
	HTParms.pCoderData = Seg->pCoderData;
	
	flags = DecodeByte(pDec);

	HTParms.HDMMR = getBIT(flags,0);
	HTParms.HDTEMPLATE = getBITS(flags,1,2);
	if ((HTParms.HDTEMPLATE)&&(HTParms.HDMMR))
		LogWarning("Invalid HDTEMPLATE");
	
	if (getBITS(flags,3,7))
		LogWarning("Invalid Reserved Bits");

	Seg->HTTable.PatWidth = DecodeByte(pDec);
	Seg->HTTable.PatHeight = DecodeByte(pDec);
	Seg->HTTable.Size = DecodeQuad(pDec) + 1;
	
	Seg->HTTable.aPatterns = jb2malloc(Seg->HTTable.Size * sizeof(BMP));

	if (HTParms.HDMMR==0) {
		MQInitDec(&HTParms.pCoderData->MQDec);
		ResetMQBuffer(&HTParms.pCoderData->Generic,MQCONTEXTMAX);
	}

	DecodeHalftoneDictionary(&HTParms,&Seg->HTTable);

}

void DecodeHalftoneRegionSegment(SEGMENTHEADER *Seg) {

	HALFTONEREGIONPARMS HTRParms;
	ULONG flags;

	DECODER *pDec = &Seg->pCoderData->Dec;

	DecodeRegionSegment(pDec,&Seg->RegSegment,Seg->bUnknownLength);
	CreateBMP(&Seg->Region,Seg->RegSegment.Width,Seg->RegSegment.Height,0);

	HTRParms.pCoderData = Seg->pCoderData;
	
	flags = DecodeByte(pDec);

	HTRParms.HMMR = getBIT(flags,0);
	HTRParms.HTEMPLATE = getBITS(flags,1,2);
	if ((HTRParms.HTEMPLATE)&&(HTRParms.HMMR))
		LogWarning("Invalid HTEMPLATE");

	HTRParms.HENABLESKIP = getBIT(flags,3);
	if ((HTRParms.HENABLESKIP)&&(HTRParms.HMMR))
		LogWarning("Invalid HENABLESKIP");

	HTRParms.HCOMBOP = getBITS(flags,4,6);
	HTRParms.HDEFPIXEL = getBIT(flags,7);

	HTRParms.HGW = DecodeQuad(pDec);
	HTRParms.HGH = DecodeQuad(pDec);
	HTRParms.HGX = (SLONG) DecodeQuad(pDec);
	HTRParms.HGY = (SLONG) DecodeQuad(pDec);
	HTRParms.HRX = DecodeBits(pDec,16);
	HTRParms.HRY = DecodeBits(pDec,16);

	if (Seg->bRefHalfToneDict==0) {
		LogError("Halftone Region Refers to No Dictionary");
	} else {
		HTRParms.HPATS = &Seg->pHTDSegment->HTTable;
		ResetMQBuffer(&HTRParms.pCoderData->Generic,MQCONTEXTMAX);		
		DecodeHalftoneRegion(&HTRParms, &Seg->Region);

		if (Seg->Type != 20) {
			RegionToPage(Seg);
			DestroyBMP(&Seg->Region);
		}

	}
}

