/*

	decmain.c - Main File for the Power JBIG-2 Decoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/

#include "jb2common.h"
#include "deccommon.h"

/* Simple routine which loads a file into ram */
void ReadFile(const char *pFileName, BYTE **pFile, ULONG *lFileSize) {
	FILE *FileIn;
	FileIn = fopen(pFileName,"rb");
	if(FileIn==NULL) {
		fprintf(stderr,"Could not load %s\n",pFileName);
		exit(1);
	}
	fseek(FileIn,0L,SEEK_END);
	*lFileSize = ftell(FileIn);
	*pFile = jb2malloc(*lFileSize);
	rewind(FileIn);
	fread(*pFile,*lFileSize,1,FileIn);
	fclose(FileIn);
}

int main(int argc, char *argv[ ]) {
	
	char sFileNameIN[1024];
	char sFileNameOUT[1024];

	JB2FILE JB2File;

	ULONG j;

	if (argc == 1) {
		LogError("Syntax: jb2dec FILENAME (without .jb2 extension)");
		exit(0);
	}

	strcpy(sFileNameIN,argv[1]);
	strcat(sFileNameIN,".jb2");
	
	/* Load the file into ram */
	ReadFile(sFileNameIN,&JB2File.pFile,&JB2File.FileLength);

	/* Parse the file and build the JB2File structure */
	if (JB2ParseFile(&JB2File)) {
		LogError ("Invalid .JB2 File");
		exit(1);
	}

	/* Decode any global (page=0) segments, tables & dictionaries */
	JB2DecodeGlobal(&JB2File);

	/* For each Page */
	for (j=0;j<JB2File.numPages;j++) {
		
		/* The BMP structure (and memory) must be set up beforehand */
		CreateBMP(&JB2File.Pages[j].PageImage,JB2File.Pages[j].PageSeg.Width,JB2File.Pages[j].PageSeg.Height,JB2File.Pages[j].PageSeg.DefPixel);

		/* Decode the specific page, store the result in Pages[j].PageImage */
		JB2DecodePage(&JB2File.Pages[j]);
	
		/* Save the pages as filename_1.bmp, filename_2.bmp, etc... */
		sprintf(sFileNameOUT,"%s_%d.bmp",argv[1],(int)JB2File.Pages[j].Number);
		SaveBMP(sFileNameOUT,&JB2File.Pages[j].PageImage);

		/* Free the memory allocated for the page */
		DestroyBMP(&JB2File.Pages[j].PageImage);
	} 
	
	/* Frees up any memory created in JB2ParseFile() or JB2DecodeGlobal() */
	JB2FreeFile(&JB2File);

	/* Frees up the memory allocated fo the file */
	jb2free(JB2File.pFile);

	return(0);
}
