/*

	decsymbols.c - Symbol Coding for the Power JBIG-2 Decoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/

#include "jb2common.h"
#include "deccommon.h"

/* A RefDict is a strucutre that represents the combination of all the 
referred to dictionary segments. */

static void BuildRefDict(REFSYMBOLDICT *RefDict, SEGMENTHEADER *MainSeg) {

	ULONG k;

	RefDict->Size = 0;
	RefDict->numRef = MainSeg->numRefSymbolDict;
	if (RefDict->numRef)
		RefDict->apDict = jb2malloc(RefDict->numRef * sizeof(*RefDict->apDict));
	for (k=0;k<RefDict->numRef;k++) {
		RefDict->apDict[k] = &MainSeg->apSDSegments[k]->SymbolDict;
		RefDict->Size += RefDict->apDict[k]->Size;
	}
}

/* Frees the RefDict structure */

static void DestroyRefDict(REFSYMBOLDICT *RefDict) {
	if (RefDict->numRef)
		jb2free(RefDict->apDict);
}

/* GetRefSymbol returns the correct symbol when dictionaries are combined */

static BMP *GetRefSymbol(REFSYMBOLDICT *RefDict, ULONG ID) {
	ULONG j = 0;
	ULONG BaseID = 0;
	
	if (ID > RefDict->Size) {
		LogErrorV("Reference Symbol ID Too Large",ID);
		return(0);
	} else {
		while (ID > BaseID+RefDict->apDict[j]->Size - 1) {
			BaseID += RefDict->apDict[j]->Size;
			j++;
		}
		return (&RefDict->apDict[j]->aSymbols[ID - BaseID]);
	}

}

static void DecodeSymbolRegion(SYMBOLREGIONPARMS *inParms, BMP *SBREG) {

	SLONG STRIPT;
	SLONG FIRSTS;
	ULONG NINSTANCES;
	SLONG DT;
	SLONG DFS;
	SLONG CURS;
	ULONG CURT;
	SLONG Si;
	SLONG Ti;
	ULONG IDi;
	BMP IBi;
	SLONG IDS;
	BOOLN Ri;
	SLONG RDWi;
	SLONG RDHi;
	SLONG RDXi;
	SLONG RDYi;
	BMP IBOi;

	ULONG RefBMPDataSize;
	BYTE *RefBMPPtr;
	GENREFREGIONPARMS GRRParms;
	ULONG X;
	ULONG Y;
	BOOLN OOB;

	DECODER *pDec = &inParms->pCoderData->Dec;
	MQINTSTRUCT *IAINT = inParms->pCoderData->aIAINT;
	
	GRRParms.pCoderData = inParms->pCoderData;

	GRRParms.GRTEMPLATE = inParms->SBRTEMPLATE;
	GRRParms.TPRON = 0;
	GRRParms.GRATX[0] = inParms->SBRATX[0];
	GRRParms.GRATX[1] = inParms->SBRATX[1];
	GRRParms.GRATY[0] = inParms->SBRATY[0];
	GRRParms.GRATY[1] = inParms->SBRATY[1];

	if (inParms->SBHUFF)
		STRIPT = DecodeHuff(pDec, inParms->pSBHUFFDT,&OOB);
	else
		STRIPT = MQDecodeInt(&IAINT[MQIADT],&OOB);
	
	STRIPT = -STRIPT * inParms->SBSTRIPS;

	FIRSTS = 0;
	NINSTANCES = 0;

	while (NINSTANCES < inParms->SBNUMINSTANCES) {

		if (inParms->SBHUFF)
			DT = DecodeHuff(pDec,inParms->pSBHUFFDT,&OOB);
		else
			DT = MQDecodeInt(&IAINT[MQIADT],&OOB);
		STRIPT = STRIPT + DT * inParms->SBSTRIPS;

		if (inParms->SBHUFF)
			DFS = DecodeHuff(pDec,inParms->pSBHUFFFS,&OOB);
		else
			DFS = MQDecodeInt(&IAINT[MQIAFS],&OOB);

		FIRSTS = FIRSTS + DFS;
		CURS = FIRSTS;

		do {
			if (inParms->SBSTRIPS > 1) {
				if (inParms->SBHUFF)
					CURT = DecodeBits(pDec, LOG2[inParms->SBSTRIPS]);
				else
					CURT = MQDecodeInt(&IAINT[MQIAIT],&OOB);
			} else {
				CURT = 0;
			}

			Ti = STRIPT + CURT;

			if (inParms->SBHUFF)
				IDi = DecodeHuffTree(pDec, inParms->pSYMCODETREE);
			else
				IDi = MQDecodeInt(&IAINT[MQIAID],&OOB);

			if (inParms->SBREFINE==0)
				Ri = 0;
			else {
				if (inParms->SBHUFF)
					Ri = DecodeBit(pDec);
				else
					Ri = MQDecodeInt(&IAINT[MQIARI],&OOB);
			}

			if (Ri==0)
				IBi = *GetRefSymbol(&inParms->SBSYMS,IDi);
			else {
				if (inParms->SBHUFF)
					RDWi = DecodeHuff(pDec, inParms->pSBHUFFRDW,&OOB);
				else
					RDWi = MQDecodeInt(&IAINT[MQIARDW],&OOB);

				if (inParms->SBHUFF)
					RDHi = DecodeHuff(pDec, inParms->pSBHUFFRDH,&OOB);
				else
					RDHi = MQDecodeInt(&IAINT[MQIARDH],&OOB);
				
				if (inParms->SBHUFF)
					RDXi = DecodeHuff(pDec, inParms->pSBHUFFRDX,&OOB);
				else
					RDXi = MQDecodeInt(&IAINT[MQIARDX],&OOB);

				if (inParms->SBHUFF)
					RDYi = DecodeHuff(pDec, inParms->pSBHUFFRDY,&OOB);
				else
					RDYi = MQDecodeInt(&IAINT[MQIARDY],&OOB);

				if (inParms->SBHUFF) {
					RefBMPDataSize = DecodeHuff(pDec, inParms->pSBHUFFRSIZE,&OOB);
					DecodeFlush(pDec);
					RefBMPPtr = DecoderPtr(pDec);
					MQInitDec(&inParms->pCoderData->MQDec);
				}
				
				IBOi = *GetRefSymbol(&inParms->SBSYMS,IDi);
				
				CreateBMP(&IBi,IBOi.Width + RDWi,IBOi.Height + RDHi,inParms->SBDEFPIXEL);

				GRRParms.GRREFERENCE = &IBOi;
				GRRParms.GRREFERENCEDX = (RDWi / 2) + RDXi;
				GRRParms.GRREFERENCEDY = (RDHi / 2) + RDYi;
				
				DecodeRefinementRegion(&GRRParms,&IBi);

				if (inParms->SBHUFF) {
					DecodeFlush(pDec);
					if ((RefBMPPtr + RefBMPDataSize)>DecoderPtr(pDec)) {
						if (RefBMPPtr + RefBMPDataSize - DecoderPtr(pDec) > 1)
							LogWarning("Skipping bits in Refinement");
						DecSkipBytes(pDec, (RefBMPPtr + RefBMPDataSize) - DecoderPtr(pDec));
					}
					else if ((RefBMPPtr + RefBMPDataSize)<DecoderPtr(pDec))
						LogWarning("Too many bits in Refinement");
				}
			}

			if ((inParms->TRANSPOSED==0)&&((inParms->REFCORNER==TOPRIGHT)||(inParms->REFCORNER==BOTTOMRIGHT))) {
				CURS = CURS + IBi.Width - 1;
			}
			if ((inParms->TRANSPOSED==1)&&((inParms->REFCORNER==BOTTOMLEFT)||(inParms->REFCORNER==BOTTOMRIGHT))) {
				CURS = CURS + IBi.Height - 1;
			}

			Si = CURS;

			if (inParms->TRANSPOSED==0)
				switch (inParms->REFCORNER) {
				case TOPLEFT:
					X = Si;
					Y = Ti;
					break;
				case BOTTOMLEFT:
					X = Si;
					Y = Ti-IBi.Height + 1;
					break;
				case TOPRIGHT:
					X = Si-IBi.Width + 1;
					Y = Ti;
					break;
				case BOTTOMRIGHT:
					X = Si-IBi.Width + 1;
					Y = Ti-IBi.Height + 1;
					break;
				}
			else
				switch (inParms->REFCORNER) {
				case TOPLEFT:
					X = Ti;
					Y = Si;
					break;
				case BOTTOMLEFT:
					X = Ti;
					Y = Si-IBi.Height + 1;
					break;
				case TOPRIGHT:
					X = Ti-IBi.Width + 1;
					Y = Si;
					break;
				case BOTTOMRIGHT:
					X = Ti-IBi.Width + 1;
					Y = Si-IBi.Height + 1;
					break;
				}

			CombineBMP(SBREG,&IBi,X,Y,inParms->SBCOMBOP);

			if(Ri)
				DestroyBMP(&IBi);

			if ((inParms->TRANSPOSED==0)&&((inParms->REFCORNER==TOPLEFT)||(inParms->REFCORNER==BOTTOMLEFT)))
				CURS = CURS + IBi.Width - 1;
			if ((inParms->TRANSPOSED==1)&&((inParms->REFCORNER==TOPLEFT)||(inParms->REFCORNER==TOPRIGHT)))
				CURS = CURS + IBi.Height - 1;

			NINSTANCES += 1;

			if (inParms->SBHUFF)
				IDS = DecodeHuff(pDec, inParms->pSBHUFFDS,&OOB);
			else
				IDS = MQDecodeInt(&IAINT[MQIADS],&OOB);
			
			if (OOB==0)
				CURS = CURS + IDS + inParms->SBDSOFFSET;

		} while (OOB==0);
	}
}

/* Returns either the standard huffman table, or the correct custom huffman 
table, based upon the bits set in the segment header */

static void GetHuffTable(HUFFTABLE *HuffStd, HUFFTABLE **pHUFF, ULONG ParmID, ULONG CustomID, ULONG def0, ULONG def1, ULONG def2, SEGMENTHEADER *Seg,ULONG *numCustom) {
	ULONG TableID = 0;

	if (ParmID==0)
			TableID = def0;
	if (ParmID==1)
			TableID = def1;
	if (ParmID==2)
			TableID = def2;

	if (TableID==0) {
		if (ParmID==CustomID) {
			if (*numCustom >= Seg->numRefCustTables)
				LogError("No Custom Table Available");
			else {
				*pHUFF = &Seg->apCTSegments[*numCustom]->CustTable;
				(*numCustom)++;
			}
		} else {
			LogError("Invalid Setting for a Huffman Table, using default");
			*pHUFF = HuffStdTable(def0);
		}
	} else
		*pHUFF = HuffStdTable(TableID);
}

void DecodeSymbolRegionSegment(SEGMENTHEADER *Seg) {

	SYMBOLREGIONPARMS SRParms;
	ULONG flags;

	ULONG *SBSYMCODELENS;
	ULONG *SBSYMCODES;

	ULONG RUNCODELEN[36];
	ULONG RUNCODES[36];
	HUFFNODE *RunTree;

	ULONG j;
	ULONG curSYMB;
	ULONG loopNum;
	ULONG Row;

	ULONG numCustom = 0;

	DECODER *pDec = &Seg->pCoderData->Dec;
	HUFFTABLE *HuffStd = Seg->pCoderData->aHuffStd;

	DecodeRegionSegment(pDec,&Seg->RegSegment,Seg->bUnknownLength);
	CreateBMP(&Seg->Region,Seg->RegSegment.Width,Seg->RegSegment.Height,0);

	BuildRefDict(&SRParms.SBSYMS,Seg);

	BITSREQUIRED(SRParms.SBSYMS.Size,SRParms.SBSYMCODELEN);

	SRParms.pCoderData = Seg->pCoderData;

	flags = DecodeBits(pDec, 16);

	SRParms.SBHUFF = getBIT(flags,0);
	SRParms.SBREFINE = getBIT(flags,1);
	SRParms.SBSTRIPS = INVLOG2[getBITS(flags,2,3)];
	SRParms.REFCORNER = getBITS(flags,4,5);
	SRParms.TRANSPOSED = getBIT(flags,6);
	SRParms.SBCOMBOP = getBITS(flags,7,8);
	SRParms.SBDEFPIXEL = getBIT(flags,9);
	SRParms.SBDSOFFSET = (signed char) getBITS(flags,10,14);
	if (SRParms.SBDSOFFSET & 0x10)
		SRParms.SBDSOFFSET |= 0xE0;
	SRParms.SBRTEMPLATE = getBIT(flags,15);
	if ((SRParms.SBRTEMPLATE)&&(SRParms.SBHUFF))
		LogWarning("Invalid SBRTEMPLATE");

	if (SRParms.SBDEFPIXEL)
		InvertBMP(&Seg->Region);

	if (SRParms.SBHUFF) {
		flags = DecodeBits(pDec, 16);
		GetHuffTable(HuffStd, &SRParms.pSBHUFFFS,getBITS(flags,0,1),3,6,7,0,Seg,&numCustom);
		GetHuffTable(HuffStd, &SRParms.pSBHUFFDS,getBITS(flags,2,3),3,8,9,10,Seg,&numCustom);
		GetHuffTable(HuffStd, &SRParms.pSBHUFFDT,getBITS(flags,4,5),3,11,12,13,Seg,&numCustom);
		GetHuffTable(HuffStd, &SRParms.pSBHUFFRDW,getBITS(flags,6,7),3,14,15,0,Seg,&numCustom);
		GetHuffTable(HuffStd, &SRParms.pSBHUFFRDH,getBITS(flags,8,9),3,14,15,0,Seg,&numCustom);
		GetHuffTable(HuffStd, &SRParms.pSBHUFFRDX,getBITS(flags,10,11),3,14,15,0,Seg,&numCustom);
		GetHuffTable(HuffStd, &SRParms.pSBHUFFRDY,getBITS(flags,12,13),3,14,15,0,Seg,&numCustom);
		GetHuffTable(HuffStd, &SRParms.pSBHUFFRSIZE,getBIT(flags,14),1,1,0,0,Seg,&numCustom);
	}
		
	if ((SRParms.SBREFINE==1)&&(SRParms.SBRTEMPLATE==0)) {
		SRParms.SBRATX[0] = (signed char) DecodeByte(pDec);
		SRParms.SBRATY[0] = (signed char) DecodeByte(pDec);
		SRParms.SBRATX[1] = (signed char) DecodeByte(pDec);
		SRParms.SBRATY[1] = (signed char) DecodeByte(pDec);
	}

	SRParms.SBNUMINSTANCES = DecodeQuad(pDec);

	if (SRParms.SBHUFF) {

		SBSYMCODELENS = jb2malloc(SRParms.SBSYMS.Size * sizeof(*SBSYMCODELENS));
		SBSYMCODES = jb2malloc(SRParms.SBSYMS.Size * sizeof(*SBSYMCODES));

		for (j=0;j<35;j++) {
			RUNCODELEN[j] = DecodeBits(pDec,4);
		}
		HuffGetPrefCodes(RUNCODELEN,RUNCODES,35);

		BuildHuffTree(&RunTree,RUNCODES,RUNCODELEN,35);

		curSYMB = 0;
		loopNum = 0;
		while (curSYMB < SRParms.SBSYMS.Size) {
			Row = DecodeHuffTree(pDec, RunTree);
			if (Row<=31) {
				SBSYMCODELENS[curSYMB] = Row;
				curSYMB ++;
			} else if (Row==32) {
				loopNum = DecodeBits(pDec, 2) + 3;
				for (j=0;j<loopNum;j++) {
					SBSYMCODELENS[curSYMB] = SBSYMCODELENS[curSYMB-1];
					curSYMB++;
				}
			} else if (Row==33) {
				loopNum = DecodeBits(pDec, 3) + 3;
				for (j=0;j<loopNum;j++) {
					SBSYMCODELENS[curSYMB] = 0;
					curSYMB++;
				}
			} else { 
				loopNum = DecodeBits(pDec, 7) + 11;
				for (j=0;j<loopNum;j++) {
					SBSYMCODELENS[curSYMB] = 0;
					curSYMB++;
				}
			}
		}

		DecodeFlush(pDec);

		HuffGetPrefCodes(SBSYMCODELENS,SBSYMCODES,SRParms.SBSYMS.Size);
		
		BuildHuffTree(&SRParms.pSYMCODETREE,SBSYMCODES,SBSYMCODELENS,SRParms.SBSYMS.Size);
	}
	
	
	if (SRParms.SBHUFF==0) {
		CreateMQInt(SRParms.pCoderData->aIAINT,SRParms.SBSYMCODELEN,&SRParms.pCoderData->MQDec);
		MQInitDec(&SRParms.pCoderData->MQDec);
	}
	ResetMQBuffer(&SRParms.pCoderData->Generic,MQCONTEXTMAX);
	ResetMQBuffer(&SRParms.pCoderData->Refine,MQCONTEXTMAX);

	DecodeSymbolRegion(&SRParms,&Seg->Region);

	if (Seg->Type != 4) {
		RegionToPage(Seg);
		DestroyBMP(&Seg->Region);
	}

	if (SRParms.SBHUFF) {
		FreeHuffTree(RunTree);
		FreeHuffTree(SRParms.pSYMCODETREE);
		jb2free(SBSYMCODELENS);
		jb2free(SBSYMCODES);
	} else {
		DestroyMQInt(SRParms.pCoderData->aIAINT);
	}
	
	DestroyRefDict(&SRParms.SBSYMS);
}


static void DecodeSymbolDictionary(SYMBOLDICTPARMS *inParms, BMP **SDEXSYMS) {

	ULONG *SDNEWSYMWIDTHS;
	ULONG HCHEIGHT;
	SLONG HCDH;
	ULONG SYMWIDTH;
	ULONG TOTWIDTH;
	ULONG HCFIRSTSYM;
	SLONG DW;
	BMP Bhc;
	ULONG i;
	ULONG j;
	ULONG REFAGGNINST;
	BOOLN *EXFLAGS;
	ULONG EXINDEX;
	BOOLN CUREXFLAG;
	ULONG EXRUNLENGTH;

	BOOLN OOB;
	ULONG *SYMCODELENS;
	ULONG *SYMCODES;
	ULONG BMSIZE;
	BYTE *MMRPtr;

	SYMBOLDICT NEWSYMS;
	
	HUFFTABLE *HuffStd = inParms->pCoderData->aHuffStd;
	DECODER *pDec = &inParms->pCoderData->Dec;
	MQINTSTRUCT *IAINT = inParms->pCoderData->aIAINT;
	
	GENREGIONPARMS GRParms;
	SYMBOLREGIONPARMS SRParms;

	GRParms.pCoderData = inParms->pCoderData;
	SRParms.pCoderData = inParms->pCoderData;

	SRParms.SBSYMS.numRef = inParms->SDINSYMS.numRef + 1;
	SRParms.SBSYMS.apDict = jb2malloc(SRParms.SBSYMS.numRef * sizeof (*SRParms.SBSYMS.apDict));
	for (j=0;j<inParms->SDINSYMS.numRef;j++)
		SRParms.SBSYMS.apDict[j] = inParms->SDINSYMS.apDict[j];
	SRParms.SBSYMS.apDict[SRParms.SBSYMS.numRef-1] = &NEWSYMS;

	NEWSYMS.aSymbols = jb2malloc(inParms->SDNUMNEWSYMS * sizeof (*NEWSYMS.aSymbols));

	EXFLAGS = jb2malloc((inParms->SDNUMNEWSYMS + inParms->SDINSYMS.Size) * sizeof(*EXFLAGS));

	if ((inParms->SDHUFF) && (inParms->SDREFAGG==0))
		SDNEWSYMWIDTHS = jb2malloc(inParms->SDNUMNEWSYMS * sizeof(*SDNEWSYMWIDTHS));

	*SDEXSYMS = jb2malloc(inParms->SDNUMEXSYMS * sizeof(**SDEXSYMS));

	if (inParms->SDHUFF) {
		SYMCODELENS = jb2malloc((inParms->SDNUMNEWSYMS + inParms->SDINSYMS.Size) * sizeof(*SYMCODELENS));
		SYMCODES = jb2malloc((inParms->SDNUMNEWSYMS + inParms->SDINSYMS.Size) * sizeof(*SYMCODES));
	}


	GRParms.MMR = 0;
	GRParms.GBTEMPLATE = inParms->SDTEMPLATE;
	GRParms.TPDON = 0;
	GRParms.USESKIP = 0;
	for (j = 0; j < 4; j++) {
		GRParms.GBATX[j] = inParms->SDATX[j];
		GRParms.GBATY[j] = inParms->SDATY[j];
	}
	GRParms.bUnknownMMRLength = 0;

	SRParms.SBHUFF = inParms->SDHUFF;
	SRParms.SBREFINE = 1;
	SRParms.SBSTRIPS = 1;

	BITSREQUIRED(inParms->SDINSYMS.Size + inParms->SDNUMNEWSYMS,SRParms.SBSYMCODELEN);

	if (inParms->SDHUFF) {
		for (j=0;j<inParms->SDNUMNEWSYMS + inParms->SDINSYMS.Size;j++) {
			SYMCODELENS[j] = SRParms.SBSYMCODELEN;
			SYMCODES[j] = j;
		}
		BuildHuffTree(&SRParms.pSYMCODETREE,SYMCODES,SYMCODELENS,inParms->SDNUMNEWSYMS + inParms->SDINSYMS.Size);
	} else {
		CreateMQInt(IAINT,SRParms.SBSYMCODELEN,&inParms->pCoderData->MQDec);
	}
		
	SRParms.SBDEFPIXEL = 0;
	SRParms.SBCOMBOP = COMBOPOR;
	SRParms.TRANSPOSED = 0;
	SRParms.REFCORNER = TOPLEFT;
	SRParms.SBDSOFFSET = 0;
	SRParms.pSBHUFFFS = HuffStdTable(6);
	SRParms.pSBHUFFDS = HuffStdTable(8);
	SRParms.pSBHUFFDT = HuffStdTable(11);
	SRParms.pSBHUFFRDW = HuffStdTable(15);
	SRParms.pSBHUFFRDH = HuffStdTable(15);
	SRParms.pSBHUFFRDX = HuffStdTable(15);
	SRParms.pSBHUFFRDY = HuffStdTable(15);
	SRParms.pSBHUFFRSIZE = HuffStdTable(1);

	SRParms.SBRTEMPLATE = inParms->SDRTEMPLATE;
	SRParms.SBRATX[0] = inParms->SDRATX[0];
	SRParms.SBRATX[1] = inParms->SDRATX[1];
	SRParms.SBRATY[0] = inParms->SDRATY[0];
	SRParms.SBRATY[1] = inParms->SDRATY[1];

	HCHEIGHT = 0;
	NEWSYMS.Size = 0;
	
	while (NEWSYMS.Size < inParms->SDNUMNEWSYMS) {
		
		if (inParms->SDHUFF)
			HCDH = DecodeHuff(pDec, inParms->pSDHUFFDH,&OOB);
		else
			HCDH = MQDecodeInt(&IAINT[MQIADH],&OOB);

		HCHEIGHT += HCDH;
		SYMWIDTH = 0;
		TOTWIDTH = 0;
		HCFIRSTSYM = NEWSYMS.Size;

		OOB = 0;
		while (OOB==0) {

			if (inParms->SDHUFF)
				DW = DecodeHuff(pDec, inParms->pSDHUFFDW,&OOB);
			else
				DW = MQDecodeInt(&IAINT[MQIADW],&OOB);

			if (OOB==0) {
				
				SYMWIDTH = SYMWIDTH + DW;
				TOTWIDTH = TOTWIDTH + SYMWIDTH;

				if ((inParms->SDHUFF==0)||(inParms->SDREFAGG==1)) {
					CreateBMP(&NEWSYMS.aSymbols[NEWSYMS.Size],SYMWIDTH,HCHEIGHT,0);
					if (inParms->SDREFAGG==0)
						DecodeGenericRegion(&GRParms,&NEWSYMS.aSymbols[NEWSYMS.Size]);
					else {
						if (inParms->SDHUFF)
							REFAGGNINST = DecodeHuff(pDec, inParms->pSDHUFFAGGINST,&OOB);
						else
							REFAGGNINST = MQDecodeInt(&IAINT[MQIAAI],&OOB);
						
						SRParms.SBNUMINSTANCES = REFAGGNINST;

						SRParms.SBSYMS.Size = inParms->SDINSYMS.Size + NEWSYMS.Size;

						DecodeSymbolRegion(&SRParms,&NEWSYMS.aSymbols[NEWSYMS.Size]);
					}
				} else {
					SDNEWSYMWIDTHS[NEWSYMS.Size] = SYMWIDTH;
				}
				NEWSYMS.Size ++;
			}

		}
		
		if ((inParms->SDHUFF)&&(inParms->SDREFAGG==0)) {
			
			CreateBMP(&Bhc,TOTWIDTH,HCHEIGHT,0);
			
			BMSIZE = DecodeHuff(pDec, inParms->pSDHUFFBMSIZE,&OOB);
			DecodeFlush(pDec);

			MMRPtr = DecoderPtr(pDec);
			
			if (BMSIZE==0)
				for (j=0;j<HCHEIGHT;j++)
					for (i=0;i<Bhc.MinWidthBytes;i++)
						Bhc.Bits[j * Bhc.AlignWidthBytes + i] = (BYTE) DecodeByte(pDec);
			else
				MMRDecode(pDec, &inParms->pCoderData->MMRTables, &Bhc,0);

			DecodeFlush(pDec);

			if (BMSIZE>0) {
				if ((MMRPtr + BMSIZE)>DecoderPtr(pDec)) {
					DecSkipBytes(pDec, (MMRPtr + BMSIZE) - DecoderPtr(pDec));
					LogWarning("Skipping bits in MMR Segment");
				}
				else if ((MMRPtr + BMSIZE)<DecoderPtr(pDec))
					LogWarning("Too many bits in MMR Segment");
			}

			j = 0;
			
			for (i=HCFIRSTSYM;i<NEWSYMS.Size;i++) {
				
				CreateBMP(&NEWSYMS.aSymbols[i],SDNEWSYMWIDTHS[i],HCHEIGHT,0);
				ExtractRegion (&NEWSYMS.aSymbols[i],&Bhc,j,0);
				j += SDNEWSYMWIDTHS[i];

			}

			DestroyBMP(&Bhc);
		}
	}	

	SRParms.SBSYMS.Size = inParms->SDINSYMS.Size + NEWSYMS.Size;

	EXINDEX = 0;
	CUREXFLAG = 0;

	while (EXINDEX < inParms->SDINSYMS.Size + inParms->SDNUMNEWSYMS) {
		if (inParms->SDHUFF)
			EXRUNLENGTH = DecodeHuff(pDec, HuffStdTable(1),&OOB);
		else 
			EXRUNLENGTH = MQDecodeInt(&IAINT[MQIAEX],&OOB);

		if (EXRUNLENGTH > 0) {
			for (j = EXINDEX; j < EXINDEX + EXRUNLENGTH; j++)
				EXFLAGS[j] = CUREXFLAG;
		}
		EXINDEX = EXINDEX + EXRUNLENGTH;
		CUREXFLAG = 1 - CUREXFLAG;
	}

	j = 0;
	for (i=0;i < inParms->SDINSYMS.Size + inParms->SDNUMNEWSYMS; i++)
		if (EXFLAGS[i]) {
			CopyBMP(&(*SDEXSYMS)[j],GetRefSymbol(&SRParms.SBSYMS,i));
			j++;
		}
	
	jb2free(EXFLAGS);
	
	if ((inParms->SDHUFF) && (inParms->SDREFAGG==0))
		jb2free(SDNEWSYMWIDTHS);
	
	if (inParms->SDHUFF) {
		FreeHuffTree(SRParms.pSYMCODETREE);
		jb2free(SYMCODELENS);
		jb2free(SYMCODES);
	} else {
		DestroyMQInt(IAINT);
	}

	for (j=0;j<inParms->SDNUMNEWSYMS;j++)
		DestroyBMP(&NEWSYMS.aSymbols[j]);

	jb2free(SRParms.SBSYMS.apDict);
	jb2free(NEWSYMS.aSymbols);
}			

void DecodeSymbolDictionarySegment(SEGMENTHEADER *Seg) {

	ULONG flags;
	ULONG j;
	SYMBOLDICTPARMS SDParms;
	ULONG numCustom = 0;

	BOOLN bUsePreviousContexts;
	BOOLN bRetainContexts;

	SEGMENTHEADER *pSeg;

	DECODER *pDec = &Seg->pCoderData->Dec;
	HUFFTABLE *HuffStd = Seg->pCoderData->aHuffStd;

	flags = DecodeBits(pDec, 16);

	SDParms.pCoderData = Seg->pCoderData;

	SDParms.SDHUFF = getBIT(flags,0);
	SDParms.SDREFAGG = getBIT(flags,1);

	if (SDParms.SDHUFF) {
		GetHuffTable(HuffStd, &SDParms.pSDHUFFDH,getBITS(flags,2,3),3,4,5,0,Seg,&numCustom);	
		GetHuffTable(HuffStd, &SDParms.pSDHUFFDW,getBITS(flags,4,5),3,2,3,0,Seg,&numCustom);	
		GetHuffTable(HuffStd, &SDParms.pSDHUFFBMSIZE,getBIT(flags,6),1,1,0,0,Seg,&numCustom);
		GetHuffTable(HuffStd, &SDParms.pSDHUFFAGGINST,getBIT(flags,7),1,1,0,0,Seg,&numCustom);
	} else
		if(getBITS(flags,2,7)!=0)
			LogWarning("Table Information with SDHUFF=0");

	bUsePreviousContexts = getBIT(flags,8);
	if ((bUsePreviousContexts)&&(SDParms.SDHUFF)&&(SDParms.SDREFAGG==0))
		LogWarning("Invalid bUsePreviousContexts");

	bRetainContexts = getBIT(flags,9);
	if ((bRetainContexts)&&(SDParms.SDHUFF)&&(SDParms.SDREFAGG==0))
		LogWarning("Invalid bRetainContexts");

	SDParms.SDTEMPLATE = getBITS(flags,10,11);
	if ((SDParms.SDTEMPLATE)&&(SDParms.SDHUFF))
		LogWarning("Invalid SDTEMPLATE");

	SDParms.SDRTEMPLATE = getBIT(flags,12);
	if ((SDParms.SDTEMPLATE)&&(SDParms.SDREFAGG))
		LogWarning("Invalid SDRTEMPLATE");

	if(getBITS(flags,13,15)!=0)
		LogWarning("Reserved Bits not 0");

	if (SDParms.SDHUFF==0) {
		if (SDParms.SDTEMPLATE==0)
			for (j=0;j<4;j++) {
				SDParms.SDATX[j] = (signed char) DecodeByte(pDec);
				SDParms.SDATY[j] = (signed char) DecodeByte(pDec);
			}
		else {
			SDParms.SDATX[0] = (signed char) DecodeByte(pDec);
			SDParms.SDATY[0] = (signed char) DecodeByte(pDec);
		}
	}
	
	if ((SDParms.SDREFAGG==1)&&(SDParms.SDRTEMPLATE==0)) {
		SDParms.SDRATX[0] = (signed char) DecodeByte(pDec);
		SDParms.SDRATY[0] = (signed char) DecodeByte(pDec);
		SDParms.SDRATX[1] = (signed char) DecodeByte(pDec);
		SDParms.SDRATY[1] = (signed char) DecodeByte(pDec);
	}

	BuildRefDict(&SDParms.SDINSYMS,Seg);

	SDParms.SDNUMEXSYMS = DecodeQuad(pDec);
	SDParms.SDNUMNEWSYMS = DecodeQuad(pDec);

	Seg->SymbolDict.Size = SDParms.SDNUMEXSYMS;

	if ((SDParms.SDHUFF==0)||(SDParms.SDREFAGG)) {
		if (bUsePreviousContexts) {
			pSeg = 0;
			for (j=0;j<Seg->numRefSymbolDict;j++)
				if (Seg->apSDSegments[Seg->numRefSymbolDict-j-1]->bStoreContextData) {
					pSeg = Seg->apSDSegments[Seg->numRefSymbolDict-j-1];
					break;
				}
			if (pSeg==0) {
				LogWarning("No Referenced Symbol Dictionaries have saved Context Data");
				ResetMQBuffer(&SDParms.pCoderData->Generic,MQCONTEXTMAX);
				ResetMQBuffer(&SDParms.pCoderData->Refine,MQCONTEXTMAX);
			} else
				for (j=0;j<MQCONTEXTMAX;j++) {
					SDParms.pCoderData->Generic.Index[j] = pSeg->MQGeneric.Index[j];
					SDParms.pCoderData->Generic.MPS[j] = pSeg->MQGeneric.MPS[j];
					SDParms.pCoderData->Refine.Index[j] = pSeg->MQRefine.Index[j];
					SDParms.pCoderData->Refine.MPS[j] = pSeg->MQRefine.MPS[j];
				}
		} else {
			ResetMQBuffer(&SDParms.pCoderData->Generic,MQCONTEXTMAX);
			ResetMQBuffer(&SDParms.pCoderData->Refine,MQCONTEXTMAX);
		}
	}

	if (SDParms.SDHUFF==0)
		MQInitDec(&SDParms.pCoderData->MQDec);

	DecodeSymbolDictionary(&SDParms,&Seg->SymbolDict.aSymbols);

	if (bRetainContexts) {
		Seg->bStoreContextData = 1;
		CreateMQBuffer(&Seg->MQGeneric,MQCONTEXTMAX);
		CreateMQBuffer(&Seg->MQRefine,MQCONTEXTMAX);
		for (j=0;j<MQCONTEXTMAX;j++) {
			pSeg->MQGeneric.Index[j] = SDParms.pCoderData->Generic.Index[j];
			pSeg->MQGeneric.MPS[j] = SDParms.pCoderData->Generic.MPS[j];
			pSeg->MQRefine.Index[j] = SDParms.pCoderData->Refine.Index[j];
			pSeg->MQRefine.MPS[j] = SDParms.pCoderData->Refine.MPS[j];
		}
	} else
		Seg->bStoreContextData = 0;

	DestroyRefDict(&SDParms.SDINSYMS);

}
