/*

	encmain.c - Main File for the Power JBIG-2 Encoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/


#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "jb2common.h"
#include "params.h"

/* JB2Encode is located in the PowerJB2enc.dll */
DLLEXPORT void JB2Encode(BMP *aPages, ULONG numPages, SECTIONPARMS *aParms, ULONG numParameters, char *FileName);

int main(int argc, char *argv[ ]) {

	BMP *Pages;
	ULONG numPages = 1; /* Note: Only 1 Page for this release */
	
	SECTIONPARMS *ParmList;
	ULONG numParameters;

	char sFileNameIN[1024];
	char sFileNameOUT[1024];

	if (argc == 1) {
		LogError("Syntax: jb2enc BMPFILE (without .bmp extension) Parameters");
		LogWarning("Visit http://spmg.ece.ubc.ca/jbig2 for instructions");
		exit(0);
	}

	numParameters = argc - 2;

	if (numParameters == 0) {
		LogWarning("No paramaters... using default settings");
		LogWarning("Visit http://spmg.ece.ubc.ca/jbig2 for instructions");
		numParameters = 1;
	}

	ParmList = jb2calloc(numParameters,sizeof(SECTIONPARMS));
	ParameterAnalysis(argc-2,&argv[2],ParmList);

	strcpy(sFileNameIN,argv[1]);
	strcat(sFileNameIN,".bmp");	
	
	strcpy(sFileNameOUT,argv[1]);
	strcat(sFileNameOUT,".jb2");

	Pages = jb2calloc(numPages,sizeof(BMP));

	if (LoadBMP(sFileNameIN, &Pages[0])) {
		fprintf(stderr,"\nERROR: Could not load file %s \n",sFileNameIN);
		exit(1);
	}	

	JB2Encode(Pages, numPages, ParmList, numParameters, sFileNameOUT);

	FreeParameters(numParameters,ParmList);
	jb2free(ParmList);

	DestroyBMP(&Pages[0]);
	jb2free(Pages);

	return(0);
}
