/*

	huffman.h - Huffman routines for the Power JBIG-2 Decoder / Encoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/

#ifndef HUFFMAN_H

#define HUFFMAN_H

#include "jb2types.h"

/* number of standard huffman tables */

#define NUMHUFFSTANDARD 15

/* just a simple macro to make code easier to read when referencing std tables */

#define HuffStdTable(A) &HuffStd[A-1]

/* The structure for a huffman decoder tree / node / leaf */

typedef struct tagHUFFNODE {
	BOOLN bEND;
	struct tagHUFFNODE *p0;
	struct tagHUFFNODE *p1;
	SLONG NodeID;
} HUFFNODE;

/* Core definition of huffman table */

typedef struct tagHUFFTABLEDEF {
	BOOLN HTOOB;
	SLONG HTLOW;
	SLONG HTHIGH;
	ULONG LOWPREFLEN;
	ULONG HIGHPREFLEN;
	ULONG OOBPREFLEN;
	ULONG numLines;
} HUFFTABLEDEF;

/* Complete definition of a huffman table */

typedef struct tagHUFFTABLE {
	HUFFTABLEDEF *pDef;
	ULONG *PREFLEN;
	ULONG *RANGELEN;
	SLONG *RANGELOW;
	ULONG *PREFCODES;
	ULONG maxPrefLen;
	HUFFNODE *Tree;
} HUFFTABLE;

void HuffGetPrefCodes(ULONG *, ULONG *, ULONG );
void BuildHuffTable(HUFFTABLE *,ULONG *,ULONG *);
void InitializeHuffStandard(HUFFTABLE *);
void EncodeHuffNum(HUFFTABLE *, SLONG, ULONG *, ULONG *, ULONG *, ULONG *);
void EncodeHuffOOB(HUFFTABLE *, ULONG *, BOOLN *);
void FreeHuffStandard(HUFFTABLE *);
void BuildHuffTree(HUFFNODE **mainnode, ULONG *, ULONG *, ULONG numCodes);
void FreeHuffTree(HUFFNODE *node);
void FreeHuffTable(HUFFTABLE *pTable);

#endif
