/*

	jb2common.h - Common routines for the Power JBIG-2 Decoder / Encoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "jb2structs.h"
#include "mqint.h"

/* #define JB2RELEASE */

#define JB2VERSION "1.0.0"

/* These macros are used to report errors & warnings - could be change here
to redirect errors to a different source */

#define LogError(A)   fprintf(stderr,"ERROR!   %s\n",A)
#define LogErrorV(A,V)  fprintf(stderr,"ERROR!   %s [%d]\n",A,(int)V)
#define LogWarning(A) fprintf(stderr,"Warning! %s\n",A)
#define LogWarningV(A,V)  fprintf(stderr,"Warning! %s [%d]\n",A,(int)V)

/* Standard min/max functions */

#define MIN(A,B) ((A<B)?(A):(B))
#define MAX(A,B) ((A>B)?(A):(B))

/* Determines the minimum number of bits B required to represent all numbers <= A */

#define BITSREQUIRED(A,B) {B = 0; while (((A)-1)>>(B)>0) (B)++;}

/* Gets & sets specific bits in a value */

#define getBIT(a,n) (((a)>>(n))&(1))
#define getBITS(a,l,h) (((a)&(0xFFFFFFFF >> (31-(h)))) >> l)
#define setBIT(a,n,v) ((a)+=((v)<<(n)))
#define setBITS(a,l,h,v) ((a)+=((v)<<l))

/* faster than (% 8) MOD8 speeds up many MOD functions */

#define MOD8(a) ((a) & 0x00000007)

extern BYTE INVLOG2[4];
extern BYTE LOG2[9];

/* maximum number of MQ contexts required for Generic coding */

#define MQCONTEXTMAX 65536

/* Number of Integer MQ coders used in JBIG-2, and thier names */

#define NUMMQINT 14

#define MQIARI 0
#define MQIAAI 1
#define MQIADH 2
#define MQIADS 3
#define MQIADT 4
#define MQIADW 5
#define MQIAEX 6
#define MQIAFS 7
#define MQIAIT 8
#define MQIARDH 9
#define MQIARDW 10
#define MQIARDX 11
#define MQIARDY 12
#define MQIAID 13

/* Reference co-ordinates for symbol segments */

#define BOTTOMLEFT 0
#define TOPLEFT 1
#define BOTTOMRIGHT 2
#define TOPRIGHT 3

void CreateMQBuffer(MQBUFFER *, ULONG );
void DestroyMQBuffer (MQBUFFER *);
void ResetMQBuffer(MQBUFFER *,ULONG);
void CreateMQInt(MQINTSTRUCT *MQInts, ULONG SYMCODELEN, void *);
void DestroyMQInt(MQINTSTRUCT *MQInts);
void ReSizeArray(void **pArray, size_t ElementSize, ULONG *ArraySize);
BOOLN ValidFileHeader(BYTE []);
ULONG Check3x3(BMP *bmp,SLONG x, SLONG y);

/* If JB2MEM is defined, then all memory allocation routines can be re-routed
here -- can be very useful in certain applications.  Otherwise, the functions
are just the regular 'c' library functions */

#ifdef JB2MEM
	void *jb2malloc( size_t size );
	void *jb2calloc( size_t num, size_t size );
	void *jb2realloc( void *memblock, size_t size );
	void jb2free( void *memblock );
	SLONG NumAllocated();
#else
	#define jb2malloc(A) malloc(A);
	#define jb2calloc(A,B) calloc(A,B);
	#define jb2realloc(A,B) realloc(A,B);
	#define jb2free(A) free(A);
#endif
