/*

	jb2structs.h - Structures used by the Power JBIG-2 Decoder / Encoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/


#ifndef JB2STRUCTS_H

#define JB2STRUCTS_H

#include "bmp.h"
#include "huffman.h"
#include "mqcoder.h"
#include "mqint.h"

typedef struct tagDECODER {
	BYTE *pDec;
	BYTE *pDecEOF;
	ULONG DecBitCount;
	BYTE DecCurrent;
	ULONG ByteCount;
} DECODER;

typedef struct tagENCODER {
	BYTE *pStart;
	BYTE *pEnc;
	ULONG EncBitCount;
	BYTE EncCurrent;
	ULONG ByteCount;
	ULONG MaxByteCount;
} ENCODER;

typedef struct tagMMRTABLES {
	HUFFNODE *WhiteTree;
	HUFFNODE *BlackTree;
	HUFFNODE *MainTree;
} MMRTABLES;

/* CODERDATA is similar to a global variables area, and is used to pass coder
structures down to routines for encoding / decoding */

typedef struct tagCODERDATA {
	DECODER Dec;
	ENCODER Enc;
	MQDEC MQDec;
	MQENC MQEnc;
	MQBUFFER Generic;
	MQBUFFER Refine;
	MQINTSTRUCT aIAINT[14];
	MMRTABLES MMRTables;
	HUFFTABLE aHuffStd[NUMHUFFSTANDARD];
} CODERDATA;

typedef struct tagSYMBOLDICT {
	ULONG Size;
	BMP *aSymbols;
} SYMBOLDICT;

typedef struct tagREFSYMBOLDICT {
	ULONG numRef;
	ULONG Size;
	SYMBOLDICT **apDict;
} REFSYMBOLDICT;

typedef struct tagHALFTONETABLE {
	ULONG Size;
	BMP *aPatterns;
	ULONG PatWidth;
	ULONG PatHeight;
} HALFTONETABLE;

typedef struct tagREGIONSEGMENT {
	ULONG Width;
	ULONG Height;
	ULONG OffsetX;
	ULONG OffsetY;
	ULONG COMBPOP;
} REGIONSEGMENT;

typedef struct tagPAGESEGMENT {
	ULONG Width;
	ULONG Height;
	ULONG ResolutionX;
	ULONG ResolutionY;
	ULONG COMBPOP;
	ULONG MaxStripeSize;
	BOOLN bAuxBuffers;
	BOOLN bOverrideOP;
	BOOLN bLossless;
	BOOLN bRefine;
	BOOLN DefPixel;
	BOOLN bStriped;
} PAGESEGMENT;

/* A SEGMENTHEADER is a key decoder structure */

typedef struct tagSEGMENTHEADER {
	ULONG Number;
	ULONG Type;

	ULONG PageNum;
	PAGESEGMENT *pPageSeg;
	BMP *pPageImage;

	BOOLN bDeferredNonRetain;
	BOOLN bRetainThis;

	ULONG DataLength;
	BYTE *pData;
	BOOLN bUnknownLength;

	REGIONSEGMENT RegSegment;
	CODERDATA *pCoderData;
	
	BMP Region;
	
	HALFTONETABLE HTTable;
	SYMBOLDICT SymbolDict;
	HUFFTABLE CustTable;

	ULONG ReferredCount;
	ULONG *aReferredNumbers;
	BOOLN *abReferredRetainFlags;

	ULONG numRefSymbolDict;
	struct tagSEGMENTHEADER **apSDSegments;

	ULONG numRefCustTables;
	struct tagSEGMENTHEADER **apCTSegments;

	BOOLN bRefIntermediate;
	struct tagSEGMENTHEADER *pINTSegment;

	BOOLN bRefHalfToneDict;
	struct tagSEGMENTHEADER *pHTDSegment;

	BOOLN bStoreContextData;
	MQBUFFER MQGeneric;
	MQBUFFER MQRefine;
} SEGMENTHEADER;

/* One of the Exported data types, JB2PAGE store the information about a page,
including the page number, the segments which refer to it, and the page image */

typedef struct tagJB2PAGE {
	ULONG Number;
	ULONG numSeg;
	SEGMENTHEADER **apSegments;
	BMP PageImage;
	PAGESEGMENT PageSeg;
} JB2PAGE;


typedef struct tagGENREGIONPARMS {
	CODERDATA *pCoderData;
	ULONG MMR;
	ULONG GBTEMPLATE;
	BOOLN TPDON;
	BOOLN USESKIP;
	BMP *pSKIP;
	SLONG GBATX[4];
	SLONG GBATY[4];
	BOOLN bUnknownMMRLength;
} GENREGIONPARMS;

typedef struct tagGENREFREGIONPARMS {
	CODERDATA *pCoderData;
	ULONG GRTEMPLATE;
	BMP *GRREFERENCE;
	SLONG GRREFERENCEDX;
	SLONG GRREFERENCEDY;
	BOOLN TPRON;
	SLONG GRATX[2];
	SLONG GRATY[2];
} GENREFREGIONPARMS;


typedef struct tagSYMBOLREGIONPARMS {
	CODERDATA *pCoderData;
	BOOLN SBHUFF;
	BOOLN SBREFINE;
	ULONG SBNUMINSTANCES;
	ULONG SBSTRIPS;
	ULONG *aSBSYMCODES;
	ULONG *aSBSYMCODELENS;
	HUFFNODE *pSYMCODETREE;
	ULONG SBSYMCODELEN;
	REFSYMBOLDICT SBSYMS;
	BOOLN SBDEFPIXEL;
	ULONG SBCOMBOP;
	BOOLN TRANSPOSED;
	ULONG REFCORNER;
	signed char SBDSOFFSET;
	ULONG SBHUFFFS;
	ULONG SBHUFFDS;
	ULONG SBHUFFDT;
	ULONG SBHUFFRDW;
	ULONG SBHUFFRDH;
	ULONG SBHUFFRDX;
	ULONG SBHUFFRDY;
	ULONG SBHUFFRSIZE;
	HUFFTABLE *pSBHUFFFS;
	HUFFTABLE *pSBHUFFDS;
	HUFFTABLE *pSBHUFFDT;
	HUFFTABLE *pSBHUFFRDW;
	HUFFTABLE *pSBHUFFRDH;
	HUFFTABLE *pSBHUFFRDX;
	HUFFTABLE *pSBHUFFRDY;
	HUFFTABLE *pSBHUFFRSIZE;
	ULONG SBRTEMPLATE;
	SLONG SBRATX[2];
	SLONG SBRATY[2];
} SYMBOLREGIONPARMS;

typedef struct tagSYMBOLDICTPARMS {
	CODERDATA *pCoderData;
	BOOLN SDHUFF;
	BOOLN SDREFAGG;
	REFSYMBOLDICT SDINSYMS;
	ULONG SDNUMNEWSYMS;
	ULONG SDNUMEXSYMS;
	ULONG SDHUFFDH;
	ULONG SDHUFFDW;
	ULONG SDHUFFBMSIZE;
	ULONG SDHUFFAGGINST;
	HUFFTABLE *pSDHUFFDH;
	HUFFTABLE *pSDHUFFDW;
	HUFFTABLE *pSDHUFFBMSIZE;
	HUFFTABLE *pSDHUFFAGGINST;
	ULONG SDTEMPLATE;
	SLONG SDATX[4];
	SLONG SDATY[4];
	ULONG SDRTEMPLATE;
	SLONG SDRATX[2];
	SLONG SDRATY[2];
	BOOLN bDirectBitmaps;
} SYMBOLDICTPARMS;

typedef struct tagHALFTONEDICTPARMS {
	CODERDATA *pCoderData;
	ULONG HDMMR;
	ULONG HDTEMPLATE;
} HALFTONEDICTPARMS;

typedef struct tagHALFTONEREGIONPARMS {
	CODERDATA *pCoderData;
	BOOLN HMMR;
	ULONG HTEMPLATE;
	HALFTONETABLE *HPATS;
	BOOLN HDEFPIXEL;
	ULONG HCOMBOP;
	BOOLN HENABLESKIP;
	ULONG HGW;
	ULONG HGH;
	SLONG HGX;
	SLONG HGY;
	ULONG HRX;
	ULONG HRY;
} HALFTONEREGIONPARMS;

typedef struct tagGRAYREGIONPARMS {
	CODERDATA *pCoderData;
	BOOLN GSMMR;
	BOOLN GSUSESKIP;
	ULONG GSBPP;
	ULONG GSW;
	ULONG GSH;
	ULONG GSTEMPLATE;
	BMP *GSKIP;
} GRAYREGIONPARMS;

/* The JB2FILE structure is the base structure for the decoder.
pFile & FileLength determine the location of the file in ram
SegmentHeaders[] is an array of all the segments in the file
Pages[] is an array of all the pages in the file
CoderData is all of the "global" memory used by the decoder */

typedef struct tagJB2FILE {
	BYTE *pFile;
	ULONG FileLength;

	ULONG numSegments;
	SEGMENTHEADER *SegmentHeaders;

	ULONG numPages;
	JB2PAGE *Pages;

	CODERDATA CoderData;

} JB2FILE;

#endif
