/*

	mqint.c - MQ Integer routines used by the Power JBIG-2 Decoder / Encoder


	Author:
		Dave Tompkins (dave.tompkins@ieee.org)
		University of British Columbia

	Source:
		http://spmg.ece.ubc.ca/jbig2
	  
	Revision History:

		1.0.0	1999-03-15	First Release


	Copyright:
		(c) 1998 University of British Columbia and Image Power, Inc.

		The program may be used for academic, demonstration and research 
		purposes only.  It may not be sold for profit or incorporated into
		any commercial programs and/or services without first obtaining a
		commercial useage license from Image Power, Inc.  This program is
		provided as is, without any express or implied warranty, including
		the warranty of fitness for a particular purpose or warranties of
		merchantability.

*/


#include "mqint.h"

#define getBIT(a,n) ((a >> n)&(1))

/* Encode a bit, and update context information */

static void MQIntEncBit(MQINTSTRUCT *inCoder, BOOLN bBit) {

	MQEncode(inCoder->pMQEnc,bBit,&inCoder->Buffer.Index[inCoder->iContext],&inCoder->Buffer.MPS[inCoder->iContext] );

	if ((inCoder->iContext < 256) || (inCoder->bIAID))
		inCoder->iContext = (inCoder->iContext << 1 ) + bBit;
	else
		inCoder->iContext = (((inCoder->iContext << 1) + bBit) & 511) | 256;
}

/* Encode an unsigned value as a string of bits */

static void MQIntEncVal(MQINTSTRUCT *inCoder, ULONG V, ULONG iBits) {
	while (iBits > 0) {
		iBits -= 1;
		MQIntEncBit(inCoder,getBIT(V,iBits));
	}
}

/* Encode the OOB value */

void MQEncodeOOB(MQINTSTRUCT *inCoder) {

	inCoder->iContext = 1;
	
	MQIntEncBit(inCoder,1);
	MQIntEncBit(inCoder,0);
	MQIntEncBit(inCoder,0);
	MQIntEncBit(inCoder,0);

}

/* Encode an Integer */

void MQEncodeInt(MQINTSTRUCT *inCoder, SLONG Val) {

	ULONG V;

	inCoder->iContext = 1;

	if (inCoder->bIAID) {
		MQIntEncVal(inCoder,(ULONG) Val,inCoder->SBSYMCODELEN);
	} else {
		if (Val < 0)
			MQIntEncBit(inCoder,1);
		else
			MQIntEncBit(inCoder,0);
		
		if (Val<0)
			V = -Val;
		else
			V = Val;

		if (V < 4) {
			MQIntEncBit(inCoder,0);
			MQIntEncVal(inCoder,V,2);
		} else {
			MQIntEncBit(inCoder,1);
			V = V - 4;
			if (V < 16) {
				MQIntEncBit(inCoder,0);
				MQIntEncVal(inCoder,V,4);
			} else {
				MQIntEncBit(inCoder,1);
				V = V - 16;
				if (V < 64) {
					MQIntEncBit(inCoder,0);
					MQIntEncVal(inCoder,V,6);
				} else {
					MQIntEncBit(inCoder,1);
					V = V - 64;
					if (V < 256) {
						MQIntEncBit(inCoder,0);
						MQIntEncVal(inCoder,V,8);
					} else {
						MQIntEncBit(inCoder,1);
						V = V - 256;
						if (V < 4096) {
							MQIntEncBit(inCoder,0);
							MQIntEncVal(inCoder,V,12);
						} else {
							MQIntEncBit(inCoder,1);
							V = V - 4096;
							MQIntEncVal(inCoder,V,32);
		}}}}}
	}
}

/* DEcode an Integer, updating context data */

static BOOLN MQIntDecBit(MQINTSTRUCT *inCoder) {

	BOOLN bBit;

	bBit = MQDecode(inCoder->pMQDec,&inCoder->Buffer.Index[inCoder->iContext],&inCoder->Buffer.MPS[inCoder->iContext]);

	if ((inCoder->iContext < 256) || (inCoder->bIAID))
		inCoder->iContext = (inCoder->iContext << 1 ) + bBit;
	else
		inCoder->iContext = (((inCoder->iContext << 1) + bBit) & 511) | 256;

	return (bBit);
}

/* Decode an unsigned value, bit by bit */

static ULONG MQIntDecVal(MQINTSTRUCT *inCoder, ULONG iBits) {

	ULONG V;

	V = 0;

	while (iBits > 0) {
		iBits -= 1;
		V <<= 1;
		V = V + MQIntDecBit(inCoder);
	}

	return (V);
}

/* Decode an Integer */

SLONG MQDecodeInt(MQINTSTRUCT *inCoder, BOOLN *bOOB) {

	SLONG V;
	ULONG S;

	*bOOB = 0;

	inCoder->iContext = 1;

	if (inCoder->bIAID) {
		V = MQIntDecVal(inCoder,inCoder->SBSYMCODELEN);
	} else {
		S = MQIntDecBit(inCoder);

		if (MQIntDecBit(inCoder)==0)
			V = MQIntDecVal(inCoder,2);
		else if (MQIntDecBit(inCoder)==0)
			V = 4 + MQIntDecVal(inCoder,4);
		else if (MQIntDecBit(inCoder)==0)
			V = 20 + MQIntDecVal(inCoder,6);
		else if (MQIntDecBit(inCoder)==0)
			V = 84 + MQIntDecVal(inCoder,8);
		else if (MQIntDecBit(inCoder)==0)
			V = 340 + MQIntDecVal(inCoder,12);
		else
			V = 4436 + MQIntDecVal(inCoder,32);

		if (S==1) {
			if (V==0)
				*bOOB = 1;
			else
				V = - V;
		}
	}

	return(V);
}
